﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/02/03 初版
//
// FV-GP440 8ch only Sample Program
// -----------------------------------------------------------------
namespace FVGP440_8chSample
{
    public partial class Form_FVGP440_8CHSAMPLE : Form
    {
        //画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_image = new List<FVIL.Data.CFviImage>();
        //FV-GP440クラス
        private List<FVIL.Video.CFviVideoGP440> m_video = new List<FVIL.Video.CFviVideoGP440>();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //エラー番号格納変数
        private int[] intErr = new int[8];
        //ビデオデバイスのオープン状態
        private bool[] m_flgVideoOpened = new bool[8];
        //フリーランの状態
        private bool m_flgFreerun;
        
        public Form_FVGP440_8CHSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

            //iniファイル選択
            fnSetIniFile();

            // 描画imageのセット
            CFviImageViewBoard0CH0.Image = m_image[0];
            CFviImageViewBoard0CH1.Image = m_image[1];
            CFviImageViewBoard0CH2.Image = m_image[2];
            CFviImageViewBoard0CH3.Image = m_image[3];
            CFviImageViewBoard1CH0.Image = m_image[4];
            CFviImageViewBoard1CH1.Image = m_image[5];
            CFviImageViewBoard1CH2.Image = m_image[6];
            CFviImageViewBoard1CH3.Image = m_image[7];

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。

            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_image.Clear();
            m_video.Clear();
            for (int Num = 0; Num < 8; Num++)
            {
                m_flgVideoOpened[Num] = false;
                m_image.Add(new FVIL.Data.CFviImage());
                m_video.Add(new FVIL.Video.CFviVideoGP440());
            }
            m_flgFreerun = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;

            //２値化コンボボックス初期化
            ComboBoxBinarize.Items.Clear();
            ComboBoxBinarize.Items.Add("グレイ");
            ComboBoxBinarize.Items.Add("２値");
            ComboBoxBinarize.SelectedIndex = 0;
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FV_PG440_1CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            for (int Num = 0; Num < 8; Num++)
            {
                if (m_flgVideoOpened[Num])
                {
                    m_video[Num].Close();
                    m_flgVideoOpened[Num] = false;
                }
            }
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイル設定画面
            Form_BROWSEINI formBroseIni = new Form_BROWSEINI();

            //iniファイルの選択
            m_strIniFilePath = "";
            if ((formBroseIni.ShowDialog(this) == DialogResult.OK))
            {
                m_strIniFilePath = formBroseIni.strIniFilePath;
            }

            if ((m_strIniFilePath.Length > 0))
            {
                int Num;
                try
                {
                    for (Num = 0; Num < 8; Num++)
                    {
                        if (m_flgVideoOpened[Num])
                        {
                            m_video[Num].Close();
                            m_flgVideoOpened[Num] = false;
                        }
                    }

                    //パラメータ設定チャンネルコンボボックスクリア
                    ComboBoxSettingCH.Items.Clear();

                    //デバイスオープン
                    int open_ch;
                    //１枚目のボード
                    for (Num = 0; Num < 4; Num++)
                    {
                        open_ch = 1 << Num;
                        if ((formBroseIni.SelectChannelNo & open_ch) != 0)
                        {
                            //デバイスオープン
                            m_video[Num].Open(formBroseIni.BoardID0, Num);
                            m_flgVideoOpened[Num] = true;
                            //取込画像を正規化
                            m_video[Num].ChangeImageSize(m_image[Num]);
                            //パラメータ設定チャンネルコンボボックス追加
                            ComboBoxSettingCH.Items.Add("BoardID:" + formBroseIni.BoardID0 + ",CH" + Num);
                        }
                    }
                    //２枚目のボード
                    for (Num = 4; Num < 8; Num++)
                    {
                        open_ch = 1 << Num;
                        if ((formBroseIni.SelectChannelNo & open_ch) != 0)
                        {
                            //デバイスオープン
                            m_video[Num].Open(formBroseIni.BoardID1, Num - 4);
                            m_flgVideoOpened[Num] = true;
                            //取込画像を正規化
                            m_video[Num].ChangeImageSize(m_image[Num]);
                            //パラメータ設定チャンネルコンボボックス追加
                            int temp = Num - 4;
                            ComboBoxSettingCH.Items.Add("BoardID:" + formBroseIni.BoardID1 + ",CH" + temp);
                        }
                    }
                    //パラメータ設定チャンネルコンボボックス
                    ComboBoxSettingCH.SelectedIndex = 0;

                    //iniファイルをコンポーネントに設定
                    for (Num = 0; Num < 8; Num++)
                    {
                        if (m_flgVideoOpened[Num])
                        {
                            m_video[Num].LoadIniFile(m_strIniFilePath);
                        }
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //ステータス表示の更新
                    //最初にオープンしたステータスを表示する
                    for (Num = 0; Num < 8; Num++)
                    {
                        if (m_flgVideoOpened[Num])
                        {
                            //カメラ露光時間の取得
                            NumericUpDownExposure.Value = m_video[Num].ExposureTime;
                            //ビデオ入力オフセットの取得
                            NumericUpDownOffset.Value = m_video[Num].VideoOffset;
                            //２値化設定の取得
                            ComboBoxBinarize.SelectedIndex = (int)m_video[Num].Binarize.Mode;
                            break;
                        }
                    }

                    //トリガ共有パラメータ構造体
                    FVIL.Video.CFviSimTrigger simtrigger = new FVIL.Video.CFviSimTrigger();
                    //ボード間トリガ共有設定パラメータ構造体
                    FVIL.Video.CFviTrgSharingMode trgsharingmode = new FVIL.Video.CFviTrgSharingMode();
                    //2値化パラメータ構造体
                    FVIL.Video.CFviVideoBinarize binarize = new FVIL.Video.CFviVideoBinarize();
                    int LastCH = 0;

                    simtrigger.SwMode = 1;          //ソフトウェアトリガを共有する
                    simtrigger.ExtMode = 1;         //外部トリガを共有する
                    for (Num = 0; Num < 8; Num++)
                    {
                        if (m_flgVideoOpened[Num])
                        {
                            //トリガモードの設定
                            m_video[Num].Trigger = 2;    //外部トリガにする
                            //同時取り込みを行う為、トリガ共有モードに設定
                            m_video[Num].SimTrigger = simtrigger;
                            //２値化モードの設定
                            binarize = m_video[Num].Binarize;
                            binarize.Mode = (uint)ComboBoxBinarize.SelectedIndex;
                            m_video[Num].Binarize = binarize;
                            //取込画像を正規化
                            m_video[Num].ChangeImageSize(m_image[Num]);
                            LastCH = Num;
                        }
                    }
                    //内部トリガが選択されている場合、最後のチャネルのみ内部トリガにする。
                    if (ComboBoxTrigger.SelectedIndex == 0)
                    {
                        m_video[LastCH].Trigger = 1;         //内部トリガにする
                    }

                    //ボード間トリガ連動設定は、ボード単位で設定するため、各ボードに対して1回ずつ設定をします。
                    //１枚目のボード
                    for (Num = 0; Num < 4; Num++)
                    {
                        if (m_flgVideoOpened[Num])
                        {
                            trgsharingmode = m_video[Num].TrgSharingMode;
                            trgsharingmode.Mode = 1;     //スレーブモードにする
                            m_video[Num].TrgSharingMode = trgsharingmode;
                            break;
                        }
                    }
                    //２枚目のボード
                    for (Num = 4; Num < 8; Num++)
                    {
                        if (m_flgVideoOpened[Num])
                        {
                            trgsharingmode = m_video[Num].TrgSharingMode;
                            trgsharingmode.Mode = 2;            //マスターモードにする
                            trgsharingmode.Ch = (uint)(LastCH - 4);     //最後のチャネルのトリガ信号に合わせて、他のボード出力にする
                            m_video[Num].TrgSharingMode = trgsharingmode;
                            break;
                        }
                    }

                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
                catch (FVIL.CFviException)
                {
                    //エラー処理
                    for (Num = 0; Num < 8; Num++)
                    {
                        intErr[Num] = m_video[Num].ErrorCode;
                    }
                    MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            int Num;
            double msec = 0;

            try
            {
                m_timer.Start();
                //時間測定開始
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        //カメラ取り込み
                        m_video[Num].GrabImageASync(m_image[Num]);
                    }
                }
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        //取り込みが完了するまで待機
                        m_video[Num].GrabWait();
                    }
                }
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");
                //再描画
                CFviImageViewBoard0CH0.Refresh();
                CFviImageViewBoard0CH1.Refresh();
                CFviImageViewBoard0CH2.Refresh();
                CFviImageViewBoard0CH3.Refresh();
                CFviImageViewBoard1CH0.Refresh();
                CFviImageViewBoard1CH1.Refresh();
                CFviImageViewBoard1CH2.Refresh();
                CFviImageViewBoard1CH3.Refresh();
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                    //画像取り込みの停止
                    if (m_flgVideoOpened[Num])
                    {
                        m_video[Num].AbortCapture();
                    }
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// フリーラン開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            if ((m_flgFreerun == true))
            {
                //ボタン表示を"Grab Continue"に変更
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
            }
            else
            {
                //ボタン表示を"Stop"に変更
                ButtonGrabContinue.Text = "Stop";
                ButtonGrabOne.Enabled = false;
                m_flgFreerun = true;
                timer1.Interval = 50;
                timer1.Start();
            }
            Application.DoEvents();

        }

        /// <summary>
        /// フリーラン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            int Num;
            double msec = 0;

            timer1.Stop();

            try
            {
                m_timer.Start();
                //時間測定開始
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        //カメラ取り込み
                        m_video[Num].GrabImageASync(m_image[Num]);
                    }
                }
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        //取り込みが完了するまで待機
                        m_video[Num].GrabWait();
                    }
                }
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");
                //再描画
                CFviImageViewBoard0CH0.Refresh();
                CFviImageViewBoard0CH1.Refresh();
                CFviImageViewBoard0CH2.Refresh();
                CFviImageViewBoard0CH3.Refresh();
                CFviImageViewBoard1CH0.Refresh();
                CFviImageViewBoard1CH1.Refresh();
                CFviImageViewBoard1CH2.Refresh();
                CFviImageViewBoard1CH3.Refresh();

                timer1.Enabled = true;

            }
            catch (FVIL.CFviException)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                    //画像取り込みの停止
                    if (m_flgVideoOpened[Num])
                    {
                        m_video[Num].AbortCapture();
                    }
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectionChangeCommitted(object sender, EventArgs e)
        {
            int Num;
            int LastCH = 0;
            try
            {
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        m_video[Num].Trigger = 2;       //外部トリガにする
                        LastCH = Num;
                    }
                }
                //内部トリガが選択されている場合、最後のチャネルのみ内部トリガにする。
                if (m_flgVideoOpened[LastCH])
                {
                    if (ComboBoxTrigger.SelectedIndex == 0)
                    {
                        m_video[LastCH].Trigger = 1;       //内部トリガにする
                    }
                }

                //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                //キャプチャボードによってこの時間は異なります。
                System.Threading.Thread.Sleep(350);
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Binarize Mode設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// グレイ画像・二値画像の入力画像選択。
        /// </remarks>
        private void ComboBoxBinarize_SelectionChangeCommitted(object sender, EventArgs e)
        {
            int Num;
            try
            {
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        //CFviVideoBinarize構造体のインスタンスのメンバを個別に変更する事は出来ない為、以下の様にして設定。
                        FVIL.Video.CFviVideoBinarize bin = m_video[Num].Binarize;
                        bin.Mode = (uint)ComboBoxBinarize.SelectedIndex;
                        m_video[Num].Binarize = bin;
                        //取込画像を正規化
                        m_video[Num].ChangeImageSize(m_image[Num]);
                    }
                }
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Setting CH コンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パラメータの取得。
        /// </remarks>
        private void ComboBoxSettingCH_SelectionChangeCommitted(object sender, EventArgs e)
        {
            int Num = 0;
            int Cnt = 0;
            try
            {
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        if (Cnt == ComboBoxSettingCH.SelectedIndex)
                        {
                            //カメラ露光時間の取得
                            NumericUpDownExposure.Value = m_video[Num].ExposureTime;
                            //ビデオ入力オフセットの取得
                            NumericUpDownOffset.Value = m_video[Num].VideoOffset;
                            //２値化閾値の取得
                            NumericUpDownThreshold.Value = (int)m_video[Num].Binarize.Threshold;
                            break;
                        }
                        Cnt++;
                    }
                }
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                intErr[0] = m_video[Num].ErrorCode;
                if(Num < 4){
                    MessageBox.Show(String.Format("ErrorCode:Board0:CH{0}={1}", Num, m_video[0].ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(String.Format("ErrorCode:Board1:CH{0}={1}", Num, m_video[0].ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

        }

        /// <summary>
        /// Video Offset値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセットの設定。
        /// </remarks>
        private void NumericUpDownOffset_ValueChanged(object sender, EventArgs e)
        {
            int Num;
            int Cnt = 0;
            try
            {
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        if (Cnt == ComboBoxSettingCH.SelectedIndex)
                        {
                            m_video[Num].VideoOffset = (int)NumericUpDownOffset.Value;
                            break;
                        }
                        Cnt++;
                    }
                }
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            int Num;
            int Cnt = 0;
            try
            {
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        if (Cnt == ComboBoxSettingCH.SelectedIndex)
                        {
                            m_video[Num].ExposureTime = (uint)NumericUpDownExposure.Value;
                            break;
                        }
                        Cnt++;
                    }
                }
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Threshold値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化しきい値の設定。
        /// </remarks>
        private void NumericUpDownThreshold_ValueChanged(object sender, EventArgs e)
        {
            int Num;
            int Cnt = 0;
            try
            {
                for (Num = 0; Num < 8; Num++)
                {
                    if (m_flgVideoOpened[Num])
                    {
                        if (Cnt == ComboBoxSettingCH.SelectedIndex)
                        {
                            //CFviVideoBinarize構造体のインスタンスのメンバを個別に変更する事は出来ない為、以下の様にして設定。
                            FVIL.Video.CFviVideoBinarize bin = m_video[Num].Binarize;
                            bin.Threshold = (uint)NumericUpDownThreshold.Value;
                            m_video[Num].Binarize = bin;
                            break;
                        }
                        Cnt++;
                    }
                }
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                for (Num = 0; Num < 8; Num++)
                {
                    intErr[Num] = m_video[Num].ErrorCode;
                }
                MessageBox.Show(String.Format("Board0:CH0={0},CH1={1},CH2={2},CH3={3}\r\nBoard1:CH0={4},CH1={5},CH2={6},CH3={7}", m_video[0].ErrorCode, m_video[1].ErrorCode, m_video[2].ErrorCode, m_video[3].ErrorCode, m_video[4].ErrorCode, m_video[5].ErrorCode, m_video[6].ErrorCode, m_video[7].ErrorCode), "ErrorCode", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。
            //表示倍率の設定。
            CFviImageViewBoard0CH0.Display.Magnification = 1.0;
            CFviImageViewBoard0CH1.Display.Magnification = 1.0;
            CFviImageViewBoard0CH2.Display.Magnification = 1.0;
            CFviImageViewBoard0CH3.Display.Magnification = 1.0;
            CFviImageViewBoard1CH0.Display.Magnification = 1.0;
            CFviImageViewBoard1CH1.Display.Magnification = 1.0;
            CFviImageViewBoard1CH2.Display.Magnification = 1.0;
            CFviImageViewBoard1CH3.Display.Magnification = 1.0;
            //再描画
            CFviImageViewBoard0CH0.Refresh();
            CFviImageViewBoard0CH1.Refresh();
            CFviImageViewBoard0CH2.Refresh();
            CFviImageViewBoard0CH3.Refresh();
            CFviImageViewBoard1CH0.Refresh();
            CFviImageViewBoard1CH1.Refresh();
            CFviImageViewBoard1CH2.Refresh();
            CFviImageViewBoard1CH3.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            CFviImageViewBoard0CH0.FitImageSize();
            CFviImageViewBoard0CH1.FitImageSize();
            CFviImageViewBoard0CH2.FitImageSize();
            CFviImageViewBoard0CH3.FitImageSize();
            CFviImageViewBoard1CH0.FitImageSize();
            CFviImageViewBoard1CH1.FitImageSize();
            CFviImageViewBoard1CH2.FitImageSize();
            CFviImageViewBoard1CH3.FitImageSize();
            //再描画
            CFviImageViewBoard0CH0.Refresh();
            CFviImageViewBoard0CH1.Refresh();
            CFviImageViewBoard0CH2.Refresh();
            CFviImageViewBoard0CH3.Refresh();
            CFviImageViewBoard1CH0.Refresh();
            CFviImageViewBoard1CH1.Refresh();
            CFviImageViewBoard1CH2.Refresh();
            CFviImageViewBoard1CH3.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = CFviImageViewBoard0CH0.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            CFviImageViewBoard0CH0.Display.Magnification = Value;
            CFviImageViewBoard0CH1.Display.Magnification = Value;
            CFviImageViewBoard0CH2.Display.Magnification = Value;
            CFviImageViewBoard0CH3.Display.Magnification = Value;
            CFviImageViewBoard1CH0.Display.Magnification = Value;
            CFviImageViewBoard1CH1.Display.Magnification = Value;
            CFviImageViewBoard1CH2.Display.Magnification = Value;
            CFviImageViewBoard1CH3.Display.Magnification = Value;
            //再描画
            CFviImageViewBoard0CH0.Refresh();
            CFviImageViewBoard0CH1.Refresh();
            CFviImageViewBoard0CH2.Refresh();
            CFviImageViewBoard0CH3.Refresh();
            CFviImageViewBoard1CH0.Refresh();
            CFviImageViewBoard1CH1.Refresh();
            CFviImageViewBoard1CH2.Refresh();
            CFviImageViewBoard1CH3.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = CFviImageViewBoard0CH0.Display.Magnification;
            Value = Value - 0.5;
            if ((Value <= 0))
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            CFviImageViewBoard0CH0.Display.Magnification = Value;
            CFviImageViewBoard0CH1.Display.Magnification = Value;
            CFviImageViewBoard0CH2.Display.Magnification = Value;
            CFviImageViewBoard0CH3.Display.Magnification = Value;
            CFviImageViewBoard1CH0.Display.Magnification = Value;
            CFviImageViewBoard1CH1.Display.Magnification = Value;
            CFviImageViewBoard1CH2.Display.Magnification = Value;
            CFviImageViewBoard1CH3.Display.Magnification = Value;
            //再描画
            CFviImageViewBoard0CH0.Refresh();
            CFviImageViewBoard0CH1.Refresh();
            CFviImageViewBoard0CH2.Refresh();
            CFviImageViewBoard0CH3.Refresh();
            CFviImageViewBoard1CH0.Refresh();
            CFviImageViewBoard1CH1.Refresh();
            CFviImageViewBoard1CH2.Refresh();
            CFviImageViewBoard1CH3.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                CFviImageViewBoard0CH0.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard0CH0.Display.OverlayAscope.Active = true;
                CFviImageViewBoard0CH1.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard0CH1.Display.OverlayAscope.Active = true;
                CFviImageViewBoard0CH2.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard0CH2.Display.OverlayAscope.Active = true;
                CFviImageViewBoard0CH3.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard0CH3.Display.OverlayAscope.Active = true;
                CFviImageViewBoard1CH0.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard1CH0.Display.OverlayAscope.Active = true;
                CFviImageViewBoard1CH1.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard1CH1.Display.OverlayAscope.Active = true;
                CFviImageViewBoard1CH2.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard1CH2.Display.OverlayAscope.Active = true;
                CFviImageViewBoard1CH3.Display.OverlayAscope.Enable = true;
                CFviImageViewBoard1CH3.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                CFviImageViewBoard0CH0.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard0CH0.Display.OverlayAscope.Active = false;
                CFviImageViewBoard0CH1.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard0CH1.Display.OverlayAscope.Active = false;
                CFviImageViewBoard0CH2.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard0CH2.Display.OverlayAscope.Active = false;
                CFviImageViewBoard0CH3.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard0CH3.Display.OverlayAscope.Active = false;
                CFviImageViewBoard1CH0.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard1CH0.Display.OverlayAscope.Active = false;
                CFviImageViewBoard1CH1.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard1CH1.Display.OverlayAscope.Active = false;
                CFviImageViewBoard1CH2.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard1CH2.Display.OverlayAscope.Active = false;
                CFviImageViewBoard1CH3.Display.OverlayAscope.Enable = false;
                CFviImageViewBoard1CH3.Display.OverlayAscope.Active = false;
            }
            //再描画
            CFviImageViewBoard0CH0.Refresh();
            CFviImageViewBoard0CH1.Refresh();
            CFviImageViewBoard0CH2.Refresh();
            CFviImageViewBoard0CH3.Refresh();
            CFviImageViewBoard1CH0.Refresh();
            CFviImageViewBoard1CH1.Refresh();
            CFviImageViewBoard1CH2.Refresh();
            CFviImageViewBoard1CH3.Refresh();

        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            //フリーラン中ならば、停止する。
	        if ((m_flgFreerun == true)) {
		        ButtonGrabContinue.Text = "Grab Continue";
		        ButtonGrabOne.Enabled = true;
		        m_flgFreerun = false;
		        timer1.Enabled = false;
		        Application.DoEvents();
	        }

        	//iniファイル選択
	        fnSetIniFile();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if ((ofdlg.ShowDialog(this) == DialogResult.OK))
            {
                try
                {
                    String name;
                    char[] ex;
                    int temp;
                    name = ".";
                    ex = name.ToCharArray();
                    //拡張子までの文字数を算出
                    temp = ofdlg.FileName.Length - ofdlg.FileName.LastIndexOfAny(ex);
                    temp = ofdlg.FileName.Length - temp;

                    //ボード０チャネル０の画像保存
                    if (m_flgVideoOpened[0] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board0CH0");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard0CH0.Image);
                    }
                    //ボード０チャネル１の画像保存
                    if (m_flgVideoOpened[1] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board0CH1");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard0CH1.Image);
                    }
                    //ボード０チャネル２の画像保存
                    if (m_flgVideoOpened[2] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board0CH2");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard0CH2.Image);
                    }
                    //ボード０チャネル３の画像保存
                    if (m_flgVideoOpened[3] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board0CH3");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard0CH3.Image);
                    }
                    //ボード１チャネル０の画像保存
                    if (m_flgVideoOpened[4] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board1CH0");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard1CH0.Image);
                    }
                    //ボード１チャネル１の画像保存
                    if (m_flgVideoOpened[5] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board1CH1");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard1CH1.Image);
                    }
                    //ボード１チャネル２の画像保存
                    if (m_flgVideoOpened[6] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board1CH2");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard1CH2.Image);
                    }
                    //ボード１チャネル３の画像保存
                    if (m_flgVideoOpened[7] == true)
                    {
                        name = ofdlg.FileName.Insert(temp, "_Board1CH3");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewBoard1CH3.Image);
                    }

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

    }
}